Package: lmomco
Type: Package
Title: L-Moments, Censored L-Moments, Trimmed L-Moments, L-Comoments,
        and Many Distributions
Version: 2.5.7
Depends: R (>= 3.5.0), utils
Imports: goftest, Lmoments, MASS
Suggests: copBasic
Date: 2026-07-03
Authors@R: person(given = "William",
                  family = "Asquith",
                  role = c("aut", "cre"),
                  email = "william.asquith@ttu.edu",
		  comment = c(ORCID = "0000-0002-7400-1861"))
Description: Extensive functions for Lmoments (LMs) and probability-weighted moments (PWMs),
  distribution parameter estimation, LMs for distributions, LM ratio diagrams, multivariate
  Lcomoments, and asymmetric (asy) trimmed LMs (TLMs). Maximum likelihood and
  maximum product spacings estimation are available. Right-tail and left-tail LM censoring
  by threshold or indicator variable are available. LMs of residual (resid) and reversed
  (rev) residual life are implemented along with 13 quantile operators for reliability analyses.
  Exact analytical bootstrap estimates of order statistics, LMs, and LM var-covars are available.
  Harri-Coble Tau34-squared Normality Test is available. Distributions with L, TL, and added
  (+) support for right-tail censoring (RC) encompass: Asy Exponential (Exp) Power [L],
  Asy Triangular [L], Cauchy [TL], Eta-Mu [L], Exp. [L], Gamma [L], Generalized (Gen) Exp
  Poisson [L], Gen Extreme Value [L], Gen Lambda [L, TL], Gen Logistic [L], Gen Normal [L],
  Gen Pareto [L+RC, TL], Govindarajulu [L], Gumbel [L], Kappa [L], Kappa-Mu [L],
  Kumaraswamy [L], Laplace [L], Linear Mean Residual Quantile Function [L], Normal [L],
  3p log-Normal [L], Pearson Type III [L], Polynomial Density-Quantile 3 and 4 [L],
  Rayleigh [L], Rev-Gumbel [L+RC], Rice [L], Singh Maddala [L], Slash [TL], 3p Student t [L],
  Truncated Exponential [L], Wakeby [L], and Weibull [L].
Maintainer: William Asquith <william.asquith@ttu.edu>
Repository: CRAN
License: GPL
NeedsCompilation: no
LazyData: true
Packaged: 2026-07-03 23:30:22 UTC; wasquith
Author: William Asquith [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7400-1861>)
Date/Publication: 2026-07-04 02:10:02 UTC
Built: R 4.4.3; ; 2026-07-04 05:49:12 UTC; unix
