importFrom("grDevices", "grey", "rgb")
importFrom("graphics", "Axis", "arrows", "axis", "axTicks", "legend", "lines", "mtext",
           "par", "plot", "points", "polygon", "text")
importFrom("stats", "approx", "binomial", "coefficients", "complete.cases",
           "cov", "dcauchy", "dexp", "dgamma", "dnorm", "dt", "glm",
           "integrate", "median", "optim", "optimize", "poly",
           "pbeta", "pcauchy", "pchisq", "pexp", "pgamma", "pnorm", "pt",
           "qbeta", "qcauchy", "qexp", "qf", "qgamma", "qnorm", "qt",
           "quantile", "runif",
           "sd", "uniroot", "var")
importFrom("utils", "packageDescription", "str")
importFrom("goftest", "pCvM")
importFrom("Lmoments", "Lmomcov")
importFrom("MASS", "mvrnorm")
###########################################################################
export(add.lmomco.axis)
export(add.log.axis)
export(Apwm2BpwmRC)
export(Bpwm2ApwmRC)
export(BEhypergeo)
export(LaguerreHalf)
export(Lcomoment.Lk12)
export(Lcomoment.Wk)
export(Lcomoment.coefficients)
export(Lcomoment.correlation)
export(Lcomoment.matrix)
export(T2prob)
export(TLmom)
export(TLmoms)
export(x2xlo)
export(xlo2qua)
export(f2f)
export(f2flo)
export(flo2f)
export(f2fpds)
export(fpds2f)
export(are.lmom.valid)
export(are.par.valid)
export(are.paraep4.valid)
export(are.parcau.valid)
export(are.parexp.valid)
export(are.pargam.valid)
export(are.pargdd.valid)
export(are.pargep.valid)
export(are.pargev.valid)
export(are.pargld.valid)
export(are.parglo.valid)
export(are.pargno.valid)
export(are.pargov.valid)
export(are.pargpa.valid)
export(are.pargum.valid)
export(are.parkap.valid)
export(are.paremu.valid)
export(are.parkmu.valid)
export(are.parkur.valid)
export(are.parlap.valid)
export(are.parlmrq.valid)
export(are.parln3.valid)
export(are.parnor.valid)
export(are.parpdq3.valid)
export(are.parpdq4.valid)
export(are.parpe3.valid)
export(are.parray.valid)
export(are.parrevgum.valid)
export(are.parrice.valid)
export(are.parsla.valid)
export(are.parsmd.valid)
export(are.parst3.valid)
export(are.partexp.valid)
export(are.partri.valid)
export(are.parwak.valid)
export(are.parwei.valid)
export(bfrlmomco)
export(cdf2lmom)
export(cdf2lmoms)
export(cdfaep4)
export(cdfcau)
export(cdfexp)
export(cdfgam)
export(cdfgdd)
export(cdfgep)
export(cdfgev)
export(cdfgld)
export(cdfglo)
export(cdfgno)
export(cdfgov)
export(cdfgpa)
export(cdfgum)
export(cdfkap)
export(cdfkmu)
export(cdfemu)
export(cdfkur)
export(cdflap)
export(cdflmrq)
export(cdfln3)
export(cdfnor)
export(cdfpdq3)
export(cdfpdq4)
export(cdfpe3)
export(cdfray)
export(cdfrevgum)
export(cdfrice)
export(cdfsla)
export(cdfsmd)
export(cdfst3)
export(cdftexp)
export(cdftri)
export(cdfwak)
export(cdfwei)
export(check.fs)
export(check.pdf)
export(cmlmomco)
export(cvm.test.lmomco)
export(disfitgovloc)
export(disfitqua)
export(dist.list)
export(dlmomco)
export(expect.max.ostat)
export(expect.min.ostat)
export(fliplmoms)
export(freq.curve.all)
export(gen.freq.curves)
export(genci)
export(genci.simple)
export(gini.mean.diff)
export(grv2prob)
export(harmonic.mean)
export(headrick.sheng.lalpha)
export(hlmomco)
export(is.aep4)
export(is.cau)
export(is.exp)
export(is.gam)
export(is.gdd)
export(is.gep)
export(is.gev)
export(is.gld)
export(is.glo)
export(is.gno)
export(is.gov)
export(is.gpa)
export(is.gum)
export(is.kap)
export(is.kur)
export(is.emu)
export(is.kmu)
export(is.lap)
export(is.lmrq)
export(is.ln3)
export(is.nor)
export(is.pdq3)
export(is.pdq4)
export(is.pe3)
export(is.ray)
export(is.revgum)
export(is.rice)
export(is.sla)
export(is.smd)
export(is.st3)
export(is.texp)
export(is.tri)
export(is.wak)
export(is.wei)
export(lalpha)
export(lcomoms2)
export(lkhlmomco)
export(lmom.ub)
export(lmom2par)
export(lmr2par)
export(lmom2pwm)
export(lmom2vec)
export(lmomRCmark)
export(lmomTLgld)
export(lmomTLgpa)
export(lmomaep4)
export(lmomcau)
export(lmomexp)
export(lmomgam)
export(lmomgdd)
export(lmomgep)
export(lmomgev)
export(lmomgld)
export(lmomglo)
export(lmomgno)
export(lmomgov)
export(lmomgpa)
export(lmomgpaRC)
export(lmomgum)
export(lmomkap)
export(lmomkur)
export(lmomemu)
export(lmomkmu)
export(lmomlap)
export(lmomlmrq)
export(lmomln3)
export(lmomnor)
export(lmompdq3)
export(lmompdq4)
export(lmompe3)
export(lmomray)
export(lmomrevgum)
export(lmomrice)
export(lmomsla)
export(lmomsmd)
export(lmomst3)
export(lmoms)
export(lmoms.cov)
export(lmoms.bootbarvar)
export(lmomsRCmark)
export(lmomtexp)
export(lmomtri)
export(lmomwak)
export(lmomwei)
export(lmorph)
export(lmrdia)
export(lmrdia46)
export(lmrdiscord)
export(lmrloc)
export(lrv2prob)
export(lrzlmomco)
export(mle2par)
export(mps2par)
export(nonexceeds)
export(eostat)
export(par2cdf)
export(par2cdf2)
export(par2lmom)
export(par2pdf)
export(par2qua)
export(par2qua2)
export(par2qua2lo)
export(par2vec)
export(parTLgld)
export(parTLgpa)
export(paraep4)
export(parcau)
export(parexp)
export(pargam)
export(pargdd)
export(pargep)
export(pargev)
export(pargld)
export(parglo)
export(pargno)
export(pargov)
export(pargpa)
export(pargpaRC)
export(pargum)
export(parkap)
export(paremu)
export(parkmu)
export(parkur)
export(parlap)
export(parlmrq)
export(parln3)
export(parnor)
export(parpdq3)
export(parpdq4)
export(parpe3)
export(parray)
export(parrevgum)
export(parrice)
export(pars2x)
export(parsla)
export(parsmd)
export(parst3)
export(partexp)
export(partri)
export(parwak)
export(parwei)
export(pdfaep4)
export(pdfcau)
export(pdfexp)
export(pdfgam)
export(pdfgdd)
export(pdfgep)
export(pdfgev)
export(pdfgld)
export(pdfglo)
export(pdfgno)
export(pdfgov)
export(pdfgpa)
export(pdfgum)
export(pdfkap)
export(pdfemu)
export(pdfkmu)
export(pdfkur)
export(pdflap)
export(pdflmrq)
export(pdfln3)
export(pdfnor)
export(pdfpdq3)
export(pdfpdq4)
export(pdfpe3)
export(pdfray)
export(pdfrevgum)
export(pdfrice)
export(pdfsla)
export(pdfsmd)
export(pdfst3)
export(pdftexp)
export(pdftri)
export(pdfwak)
export(pdfwei)
export(plmomco)
export(plotlmrdia)
export(plotlmrdia46)
export(plotradarlmr)
export(pmoms)
export(pp)
export(pp.f)
export(pp.median)
export(prettydist)
export(prob2T)
export(prob2lrv)
export(prob2grv)
export(pwm)
export(pwm.beta2alpha)
export(pwm.alpha2beta)
export(pwm.gev)
export(pwm.pp)
export(pwm.ub)
export(pwm2lmom)
export(pwm2vec)
export(pwmLC)
export(pwmRC)
export(qlmomco)
export(qua.ostat)
export(qua2ci)
export(qua2ci.cov)
export(qua2ci.simple)
export(quaaep4)
export(quacau)
export(quaexp)
export(quagam)
export(quagdd)
export(quagep)
export(quagev)
export(quagld)
export(quaglo)
export(quagno)
export(quagov)
export(quagpa)
export(quagum)
export(quakap)
export(quaemu)
export(quakmu)
export(quakur)
export(qualap)
export(qualmrq)
export(qualn3)
export(quanor)
export(quapdq3)
export(quapdq4)
export(quape3)
export(quaray)
export(quarevgum)
export(quarice)
export(quasla)
export(quasmd)
export(quast3)
export(quatexp)
export(quatri)
export(quawak)
export(quawei)
export(quaaep4kapmix)
export(ralmomco)
export(rralmomco)
export(riglmomco)
export(reslife.lmoms)
export(rreslife.lmoms)
export(rlmomco)
export(rmlmomco)
export(rmvarlmomco)
export(rrmlmomco)
export(rrmvarlmomco)
export(sen.mean)
export(sentiv.curve)
export(slmomco)
export(stttlmomco)
export(supdist)
export(tau34sq.normtest)
export(theoLmoms)
export(theoLmoms.max.ostat)
export(theoLmoms.min.ostat)
export(theoTLmoms)
export(theopwms)
export(tlmr2par)
export(tlmrcau)
export(tlmrexp)
export(tlmrgev)
export(tlmrglo)
export(tlmrgno)
export(tlmrgpa)
export(tlmrgum)
export(tlmrln3)
export(tlmrnor)
export(tlmrpe3)
export(tlmrray)
export(tttlmomco)
export(vec2TLmom)
export(vec2lmom)
export(vec2par)
export(vec2pwm)
export(x2pars)
export(z.par2cdf)
export(z.par2qua)
export(.lmomcohash)
#export(lmomcoBook)
#export(lmomcoNews)
export(dat2bernqua)
export(dat2bernquaf)
export(lmoms.bernstein)
export(pfactor.bernstein)
export(cdfben)
export(lmomben)
export(pmfben)
export(quaben)

